#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2014 Adán Mauri Ungaro <adan.mauri@gmail.com>
#
# This file is part of FrameDuino. FrameDuino is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.  See the file
# COPYING included with this distribution for more information.

import usb
import os
from abstract_driver import AbstractDriver

#ERRORS
INCORRECT_BOOTLOADER = "Incorrect bootloader version, available: 2, 4"
DEVICES_NOT_FOUND    = "No open devices found or no device connected."
DEVICE_NOT_CONNECTED = "Device is not open or no device connected."

class USBDriver(AbstractDriver):
    
    VENDOR        = 0x04D8
    PRODUCT       = 0xFEAA

    TIMEOUT       = None
    INTERFACE     = 0
    ENDPOINT_OUT  = 0x01

    BOOTLOADER    = None
    CONFIGURATION = 0x03
    ENDPOINT_IN   = 0x82
            
    DEVNAME       = None
    DEV           = None
    
    # ----------------------------------------------------------------------
    # Basic Functions
    # ----------------------------------------------------------------------
    
    def __init__(self, bootloader=4, timeout=1000, dev=None):
        self.TIMEOUT = timeout  #1000

        if bootloader == 2 or 4:
            self.setBootloader(bootloader)
        else:
            raise Exception(INCORRECT_BOOTLOADER)

        list_devs = USBDriver.avalilableDevs()
        if dev is None:
            if list_devs:
                self.DEV = list_devs[0]
            else:
                raise Exception(DEVICES_NOT_FOUND)
        else:
            if not dev in list_devs:
                raise Exception(DEVICE_NOT_CONNECTED)
            else:
                self.DEV = dev

        self.__connect__()

    # ----------------------------------------------------------------------        
    def __connect__(self):
        try:
            self.dh = self.DEV.open()
            self.dh.setConfiguration(self.CONFIGURATION)
            self.dh.claimInterface(self.INTERFACE)
        except usb.core.USBError, V:
            raise Exception(V[1])

    # ----------------------------------------------------------------------                    
    def setBootloader(self, bootloader):  
         self.BOOTLOADER = bootloader
         if (self.BOOTLOADER == 2):
            self.CONFIGURATION = 0x03
            self.ENDPOINT_IN   = 0x82
         else:
            if (self.BOOTLOADER == 4):
                self.CONFIGURATION = 0x01
                self.ENDPOINT_IN   = 0x81

    # ----------------------------------------------------------------------                    
    def getBootloader(self, bootloader):  
         return self.BOOTLOADER
    
    # ----------------------------------------------------------------------
    # I/O Functions
    # ----------------------------------------------------------------------
    
    def write(self, msg):
        try:
            self.dh.bulkWrite(self.ENDPOINT_OUT, msg, self.TIMEOUT)
        except usb.USBError, V:
            raise Exception(V[1])

    #----------------------------------------------------------------------
    def read(self, length=64):
        try:
            return "".join(map(chr, self.dh.bulkRead(self.ENDPOINT_IN, length, self.TIMEOUT)))
        except usb.core.USBError, V:
            raise Exception(V[1])

    def readProcess(self, cmd):
        self.write(cmd)
        return self.read()

    # ----------------------------------------------------------------------
    # Static Methods
    # ----------------------------------------------------------------------
    @staticmethod        
    def avalilableDevs():
        devs = []
        busses = usb.busses()
        for bus in busses:
            for dev in bus.devices:
                if dev.idVendor == USBDriver.VENDOR and dev.idProduct == USBDriver.PRODUCT:
                    devs.append(dev)
        return devs




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2014 Adán Mauri Ungaro <adan.mauri@gmail.com>
#
# This file is part of FrameDuino. FrameDuino is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.  See the file
# COPYING included with this distribution for more information.

import time
import bluetooth
import os
from abstract_driver import AbstractDriver
from pprint import pprint

#ERRORS
DEVICES_NOT_FOUND    = "Could not find bluetooth devices nearby."
DEVICE_NOT_CONNECTED = "Could not find target bluetooth device nearby."

class BluetoothDriver(AbstractDriver):
    
    DEVICE_NAME    = None
    DEVICE_ADDRESS = None
    DEVICE_PORT    = 1
    DEV            = None
    
    # ----------------------------------------------------------------------
    # Basic Functions
    # ----------------------------------------------------------------------
    
    def __init__(self, name="Pinguino", dev=None):
        self.DEVICE_NAME = name
        
        list_devs = BluetoothDriver.nearbyDevs(self.DEVICE_NAME)        
        if dev is None:
            if list_devs:
                self.DEVICE_ADDRESS = list_devs[0]
            else:
                raise Exception(DEVICES_NOT_FOUND)
        else:
            if not dev in list_devs:
                raise Exception(DEVICE_NOT_CONNECTED)
            else:
                self.DEVICE_ADDRESS = dev
        
        self.__connect__()

    # ----------------------------------------------------------------------        
    def __connect__(self):
        pprint(self.DEVICE_ADDRESS)
        while(True):    
            try:
                self.DEV = bluetooth.BluetoothSocket(bluetooth.RFCOMM)
                self.DEV.connect((self.DEVICE_ADDRESS, self.DEVICE_PORT))
                break;
            except bluetooth.btcommon.BluetoothError as error:
                self.DEV.close()
                print "Could not connect: ", error, "; Retrying in 10s..."
                time.sleep(10)
            
    # ----------------------------------------------------------------------        
    def __disconnect__(self):
        self.DEV.close()      
     
    # ----------------------------------------------------------------------        
    def disconnect(self):
        self.__disconnect__()         
               
    # ----------------------------------------------------------------------
    # I/O Functions
    # ----------------------------------------------------------------------
    
    def write(self, msg):
        self.DEV.send(msg)
        '''try:
            
        except usb.USBError, V:
            raise Exception(V[1])'''

    #----------------------------------------------------------------------
    def read(self, length=1024):
        return self.DEV.recv(length)
        '''try:
            return "".join(map(chr, self.dh.bulkRead(self.ENDPOINT_IN, length, self.TIMEOUT)))
        except usb.core.USBError, V:
            raise Exception(V[1])'''

    def readProcess(self, cmd):
        self.write(cmd)
        return self.read()
    
    # ----------------------------------------------------------------------
    # Static Methods
    # ----------------------------------------------------------------------
    @staticmethod        
    def nearbyDevs(target_name = "Pinguino"):
        devs = []
        pprint(devs)
        nearby_devices = bluetooth.discover_devices()
        pprint(nearby_devices)
        for dev in nearby_devices:
            if target_name == bluetooth.lookup_name( dev ):
                devs.append(dev)
        return devs   




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2014 Adán Mauri Ungaro <adan.mauri@gmail.com>
#
# This file is part of FrameDuino. FrameDuino is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.  See the file
# COPYING included with this distribution for more information.

import serial
import os
from abstract_driver import AbstractDriver

if os.name == "posix": #GNU/Linux
    os.environ["PORTNAME"] = "/dev/ttyACM%d"

elif os.name == "nt":  #Windows
    os.environ["PORTNAME"] = "COM%d"

#ERRORS
PORTS_NOT_FOUND      = "No open ports found or no device connected."
PORT_NOT_CONNECTED   = "Port is not open or no device connected."
NO_CDC_DEVICES       = "No CDC devices connected to USB host"

class CDCDriver(AbstractDriver):
        
    PORTNAME = None
    PORT     = None
    BAUDRATE = None
    TIMEOUT  = None
    
    cdc      = None

    # ----------------------------------------------------------------------
    # Basic Functions
    # ----------------------------------------------------------------------
        
    def __init__(self, port=None, baudrate=9600, timeout=1):        
        self.TIMEOUT  = timeout   #1
        self.BAUDRATE = baudrate  #9600
        self.PORTNAME = os.getenv("PORTNAME")

        list_ports = CDCDriver.avalilablePorts()
        if port is None:
            if list_ports:
                self.PORT = list_ports[0]
            else:
                raise Exception(PORTS_NOT_FOUND)
        else:
            if not port in list_ports:
                raise Exception(PORT_NOT_CONNECTED)
            else: self.PORT = port

        self.__connect__()

    # ----------------------------------------------------------------------        
    def __connect__(self):
        try:
            self.cdc = serial.Serial(port=self.PORTNAME%self.PORT, timeout=self.TIMEOUT, baudrate=self.BAUDRATE)
        except:
            raise Exception(NO_CDC_DEVICES)
    
    # ----------------------------------------------------------------------
    # I/O Functions
    # ----------------------------------------------------------------------
    
    def write(self, msg):
        try:
            self.cdc.write(msg)
        except serial.serialutil.SerialException, V:
            raise Exception(V[0])

    #----------------------------------------------------------------------
    def read(self, length=64):
        try:
            return self.cdc.read(length)
        except usb.USBError, V:
            raise Exception(V[0])

    def readProcess(self, cmd):
        self.write(cmd)
        return self.read()

    # ----------------------------------------------------------------------
    # Static Methods
    # ----------------------------------------------------------------------
    @staticmethod
    def avalilablePorts():
        ports = []
        for port in range(30):
            try:
                dev = serial.Serial(os.getenv("PORTNAME")%port)
                ports.append(port)
                dev.close()
            except:
                continue
        return ports




#!/usr/bin/env python
#-*- coding: utf-8 -*-

from distutils.core import setup

setup(
    name='FrameDuino',
    version='0.1',
    description='Pinguino Board Framework',
    author='Adán Mauri Ungaro',
    author_email='adan.mauri@gmail.com',
    url='https://github.com/adanmauri/frameduino',
    packages=['frameduino', 'frameduino.drivers'],
    license='GPLv3'    
)
